@@ -1,6 +1,6 @@
 
-local rawset, next, table, pairs, print, require, io, os, setmetatable =
-      rawset, next, table, pairs, print, require, io, os, setmetatable
+local rawset, next, table, pairs, print, require, io, os, setmetatable, pcall, ipairs, package =
+      rawset, next, table, pairs, print, require, io, os, setmetatable, pcall, ipairs, package
 
 --- Configuration for LuaRocks.
 -- Tries to load the user's configuration file and
@@ -9,18 +9,37 @@ local rawset, next, table, pairs, print, require, io, os, setmetatable =
 -- file format documentation</a> for details.
 module("luarocks.cfg")
 
+-- Load site-local global configurations
+local config = require("luarocks.config")
+
+program_version = "2.0.1"
+user_agent = "LuaRocks/"..program_version
+
 local persist = require("luarocks.persist")
 
+local popen_ok, popen_result = pcall(io.popen, "")
+if popen_ok then
+   if popen_result then
+      popen_result:close()
+   end
+else
+   print("Your version of Lua does not support io.popen,")
+   print("which is required by LuaRocks. Please check your Lua installation.")
+   os.exit(1)
+end
+
+-- System detection:
+
 local detected = {}
 local system,proc
 
 -- A proper installation of LuaRocks will hardcode the system
--- and proc values with LUAROCKS_UNAME_S and LUAROCKS_UNAME_M,
+-- and proc values with config.LUAROCKS_UNAME_S and config.LUAROCKS_UNAME_M,
 -- so that this detection does not run every time. When it is
 -- performed, we use the Unix way to identify the system,
 -- even on Windows (assuming UnxUtils or Cygwin).
-system = LUAROCKS_UNAME_S or io.popen("uname -s"):read("*l")
-proc = LUAROCKS_UNAME_M or io.popen("uname -m"):read("*l")
+system = config.LUAROCKS_UNAME_S or io.popen("uname -s"):read("*l")
+proc = config.LUAROCKS_UNAME_M or io.popen("uname -m"):read("*l")
 if proc:match("i[%d]86") then
    proc = "x86"
 elseif proc:match("amd64") or proc:match("x86_64") then
@@ -48,6 +67,8 @@ else
    -- Fall back to Unix in unknown systems.
 end
 
+-- Path configuration:
+
 local sys_config_file, home_config_file, home_tree
 if detected.windows then
    home = os.getenv("APPDATA") or "c:"
@@ -64,9 +85,9 @@ end
 variables = {}
 rocks_trees = {}
 
-persist.load_into_table(LUAROCKS_SYSCONFIG or sys_config_file, _M)
+persist.load_into_table(config.LUAROCKS_SYSCONFIG or sys_config_file, _M)
 
-if not LUAROCKS_FORCE_CONFIG then
+if not config.LUAROCKS_FORCE_CONFIG then
    home_config_file = os.getenv("LUAROCKS_CONFIG") or home_config_file
    local home_overrides = persist.load_into_table(home_config_file, { home = home })
    if home_overrides then
@@ -75,46 +96,67 @@ if not LUAROCKS_FORCE_CONFIG then
    end
 end
 
+if not next(rocks_trees) then
+   if home_tree then
+      table.insert(rocks_trees, home_tree)
+   end
+   if config.LUAROCKS_ROCKS_TREE then
+      table.insert(rocks_trees, config.LUAROCKS_ROCKS_TREE)
+   end
+end
+
+-- Configure defaults:
+
+local root = config.LUAROCKS_ROCKS_TREE or home_tree
 local defaults = {
+   root_dir = root,
+   rocks_dir = root.."/lib/luarocks/rocks",
+   lua_modules_path = "/share/lua/5.1/",
+   lib_modules_path = "/lib/lua/5.1/",
+   deploy_bin_dir = root.."/bin/",
+   deploy_lua_dir = root.."/share/lua/5.1/",
+   deploy_lib_dir = root.."/lib/lua/5.1/",
+
    arch = "unknown",
    lib_extension = "unknown",
    obj_extension = "unknown",
-   rocks_servers = {
-      "http://luarocks.luaforge.net/rocks"
-   },
-   lua_extension = "lua",
-   lua_interpreter = LUA_INTERPRETER or "lua",
-   downloader = LUAROCKS_DOWNLOADER or "wget",
-   md5checker = LUAROCKS_MD5CHECKER or "md5sum",
-   variables = {}
-}
 
-defaults.external_deps_subdirs = {
-   bin = "bin",
-   lib = "lib",
-   include = "include"
+   rocks_servers = {
+      "http://www.luarocks.org/repositories/rocks"
+   },
+
+   lua_extension = "lua",
+   lua_interpreter = config.LUA_INTERPRETER or "lua",
+   downloader = config.LUAROCKS_DOWNLOADER or "wget",
+   md5checker = config.LUAROCKS_MD5CHECKER or "md5sum",
+
+   variables = {},
+   
+   external_deps_subdirs = {
+      bin = "bin",
+      lib = "lib",
+      include = "include"
+   },
+   runtime_external_deps_subdirs = {
+      bin = "bin",
+      lib = "lib",
+      include = "include"
+   },
 }
-defaults.runtime_external_deps_subdirs = defaults.external_deps_subdirs
 
 if detected.windows then
    home_config_file = home_config_file:gsub("\\","/")
-   defaults.lib_extension = "dll"
-   defaults.obj_extension = "obj"
-   local rootdir = LUAROCKS_ROCKS_TREE or home_tree
-   defaults.root_dir = rootdir
-   defaults.rocks_dir = rootdir.."/rocks/"
-   defaults.scripts_dir = rootdir.."/bin/"
-   defaults.external_deps_dirs = { "c:/external/" }
-   defaults.variables.LUA_BINDIR = LUA_BINDIR and LUA_BINDIR:gsub("\\", "/") or "c:/lua5.1/bin"
-   defaults.variables.LUA_INCDIR = LUA_INCDIR and LUA_INCDIR:gsub("\\", "/") or "c:/lua5.1/include"
-   defaults.variables.LUA_LIBDIR = LUA_LIBDIR and LUA_LIBDIR:gsub("\\", "/") or "c:/lua5.1/lib"
    defaults.arch = "win32-"..proc
    defaults.platforms = {"win32", "windows" }
+   defaults.lib_extension = "dll"
+   defaults.external_lib_extension = "dll"
+   defaults.obj_extension = "obj"
+   defaults.external_deps_dirs = { "c:/external/" }
+   defaults.variables.LUA_BINDIR = config.LUA_BINDIR and config.LUA_BINDIR:gsub("\\", "/") or "c:/lua5.1/bin"
+   defaults.variables.LUA_INCDIR = config.LUA_INCDIR and config.LUA_INCDIR:gsub("\\", "/") or "c:/lua5.1/include"
+   defaults.variables.LUA_LIBDIR = config.LUA_LIBDIR and config.LUA_LIBDIR:gsub("\\", "/") or "c:/lua5.1/lib"
    defaults.cmake_generator = "MinGW Makefiles"
-   -- TODO: Split Windows flavors between mingw and msvc
-   -- defaults.make = "make"
-   -- defaults.makefile = "Makefile"
-   defaults.make = "nmake"
+   defaults.make = "nmake" -- TODO: Split Windows flavors between mingw and msvc
    defaults.makefile = "Makefile.win"
    defaults.variables.CC = "cl"
    defaults.variables.LD = "link"
@@ -131,19 +173,17 @@ if detected.windows then
       lib = { "?.dll" },
       include = { "?.h" }
    }
+   defaults.local_cache = home.."/cache/luarocks"
 end
 
 if detected.unix then
    defaults.lib_extension = "so"
+   defaults.external_lib_extension = "so"
    defaults.obj_extension = "o"
-   local rootdir = LUAROCKS_ROCKS_TREE or home_tree
-   defaults.root_dir = rootdir
-   defaults.rocks_dir = rootdir.."/rocks/"
-   defaults.scripts_dir = rootdir.."/bin/"
    defaults.external_deps_dirs = { "/usr/local", "/usr" }
-   defaults.variables.LUA_BINDIR = LUA_BINDIR or "/usr/local/bin"
-   defaults.variables.LUA_INCDIR = LUA_INCDIR or "/usr/local/include"
-   defaults.variables.LUA_LIBDIR = LUA_LIBDIR or "/usr/local/lib"
+   defaults.variables.LUA_BINDIR = config.LUA_BINDIR or "/usr/local/bin"
+   defaults.variables.LUA_INCDIR = config.LUA_INCDIR or "/usr/local/include"
+   defaults.variables.LUA_LIBDIR = config.LUA_LIBDIR or "/usr/local/lib"
    defaults.variables.CFLAGS = "-O2"
    defaults.cmake_generator = "Unix Makefiles"
    defaults.make = "make"
@@ -161,6 +201,7 @@ if detected.unix then
       lib = { "lib?.so" },
       include = { "?.h" }
    }
+   defaults.local_cache = home.."/.cache/luarocks"
 end
 
 if detected.cygwin then
@@ -173,8 +214,6 @@ if detected.cygwin then
    defaults.variables.LIBFLAG = "-shared"
 end
 
-defaults.external_lib_extension = defaults.lib_extension
-
 if detected.macosx then
    defaults.external_lib_extension = "dylib"
    defaults.arch = "macosx-"..proc
@@ -209,18 +248,12 @@ end
 defaults.variables.LUA = defaults.lua_interpreter
 defaults.variables.LIB_EXTENSION = defaults.lib_extension
 defaults.variables.OBJ_EXTENSION = defaults.obj_extension
-defaults.variables.LUAROCKS_PREFIX = LUAROCKS_PREFIX
+defaults.variables.LUAROCKS_PREFIX = config.LUAROCKS_PREFIX
 
-local cfg_mt = {
-   __index = function(t, k)
-      local default = defaults[k]
-      if default then
-         rawset(t, k, default)
-      end
-      return default
-   end
-}
+-- Use defaults:
 
+-- Populate values from 'defaults.variables' in 'variables' if they were not
+-- already set by user.
 if not _M.variables then
    _M.variables = {}
 end
@@ -230,13 +263,21 @@ for k,v in pairs(defaults.variables) do
    end
 end
 
+-- For values not set in the config file, use values from the 'defaults' table.
+local cfg_mt = {
+   __index = function(t, k)
+      local default = defaults[k]
+      if default then
+         rawset(t, k, default)
+      end
+      return default
+   end
+}
 setmetatable(_M, cfg_mt)
 
-if not next(rocks_trees) then
-   if home_tree then
-      table.insert(rocks_trees, home_tree)
-   end
-   if LUAROCKS_ROCKS_TREE then
-      table.insert(rocks_trees, LUAROCKS_ROCKS_TREE)
-   end
+
+for _,tree in ipairs(rocks_trees) do
+   package.path = tree..lua_modules_path.."/?.lua;"..tree..lua_modules_path.."/?/init.lua;"..package.path
+   package.cpath = tree..lib_modules_path.."/?."..lib_extension..";"..package.cpath
 end
+
