@@ -1,12 +1,12 @@
 
-program_version = "1.0.1"
-
 --- Functions for command-line scripts.
 module("luarocks.command_line", package.seeall)
 
 local util = require("luarocks.util")
 local cfg = require("luarocks.cfg")
 local fs = require("luarocks.fs")
+local path = require("luarocks.path")
+local dir = require("luarocks.dir")
 
 --- Display an error message and exit.
 -- @param message string: The error message.
@@ -15,7 +15,7 @@ local function die(message)
 
    local ok, err = pcall(util.run_scheduled_functions)
    if not ok then
-      print("\nLuaRocks "..program_version.." internal bug (please report at luarocks-developers@lists.luaforge.net):\n"..err)
+      print("\nLuaRocks "..cfg.program_version.." internal bug (please report at luarocks-developers@lists.luaforge.net):\n"..err)
    end
    print("\nError: "..message)
    os.exit(1)
@@ -52,16 +52,20 @@ function run_command(...)
       end
       local root_dir = fs.absolute_name(flags["to"])
       cfg.root_dir = root_dir
-      cfg.rocks_dir = root_dir.."/rocks"
-      cfg.scripts_dir = root_dir.."/bin"
+      cfg.rocks_dir = path.rocks_dir(root_dir)
+      cfg.deploy_bin_dir = path.deploy_bin_dir(root_dir)
+      cfg.deploy_lua_dir = path.deploy_lua_dir(root_dir)
+      cfg.deploy_lib_dir = path.deploy_lib_dir(root_dir)
    else
       local trees = cfg.rocks_trees
       for i = #trees, 1, -1 do
          local tree = trees[i]
          if fs.make_dir(tree) and fs.is_writable(tree) then
             cfg.root_dir = tree
-            cfg.rocks_dir = tree.."/rocks"
-            cfg.scripts_dir = rawget(cfg, "scripts_dir") or tree.."/bin"
+            cfg.rocks_dir = path.rocks_dir(tree)
+            cfg.deploy_bin_dir = rawget(cfg, "deploy_bin_dir") or path.deploy_bin_dir(tree)
+            cfg.deploy_lua_dir = rawget(cfg, "deploy_lua_dir") or path.deploy_lua_dir(tree)
+            cfg.deploy_lib_dir = rawget(cfg, "deploy_lib_dir") or path.deploy_lib_dir(tree)
             break
          end
       end
@@ -69,16 +73,18 @@ function run_command(...)
 
    cfg.root_dir = cfg.root_dir:gsub("/+$", "")
    cfg.rocks_dir = cfg.rocks_dir:gsub("/+$", "")
-   cfg.scripts_dir = cfg.scripts_dir:gsub("/+$", "")
+   cfg.deploy_bin_dir = cfg.deploy_bin_dir:gsub("/+$", "")
+   cfg.deploy_lua_dir = cfg.deploy_lua_dir:gsub("/+$", "")
+   cfg.deploy_lib_dir = cfg.deploy_lib_dir:gsub("/+$", "")
    
    cfg.variables.ROCKS_TREE = cfg.root_dir
-   cfg.variables.SCRIPTS_DIR = cfg.scripts_dir
+   cfg.variables.SCRIPTS_DIR = cfg.deploy_bin_dir
 
    if flags["from"] then
       if flags["from"] == true then
          die("Argument error: use --from=<url>")
       end
-      local protocol, path = fs.split_url(flags["from"])
+      local protocol, path = dir.split_url(flags["from"])
       table.insert(cfg.rocks_servers, 1, protocol.."://"..path)
    end
    
@@ -92,7 +98,7 @@ function run_command(...)
    local command
    
    if flags["version"] then
-      print(program_name.." "..program_version)
+      print(program_name.." "..cfg.program_version)
       print(program_description)
       print()
       os.exit(0)
@@ -118,7 +124,7 @@ function run_command(...)
    command = command:gsub("-", "_")
    if commands[command] then
       local xp, ok, err = xpcall(function() return commands[command].run(unpack(args)) end, function(err)
-         die(debug.traceback("LuaRocks "..program_version
+         die(debug.traceback("LuaRocks "..cfg.program_version
             .." bug (please report at luarocks-developers@lists.luaforge.net).\n"
             ..err, 2))
       end)
@@ -128,6 +134,5 @@ function run_command(...)
    else
       die("Unknown command: "..command)
    end
-   
    util.run_scheduled_functions()
 end
