@@ -1,107 +1,131 @@
 
---- Functions for querying and manipulating manifest files.
 module("luarocks.manif", package.seeall)
 
-local util = require("luarocks.util")
-local fs = require("luarocks.fs")
-local search = require("luarocks.search")
-local rep = require("luarocks.rep")
-local deps = require("luarocks.deps")
-local cfg = require("luarocks.cfg")
+local manif_core = require("luarocks.manif_core")
 local persist = require("luarocks.persist")
 local fetch = require("luarocks.fetch")
-local type_check = require("luarocks.type_check")
+local dir = require("luarocks.dir")
+local fs = require("luarocks.fs")
+local search = require("luarocks.search")
+local util = require("luarocks.util")
+local cfg = require("luarocks.cfg")
+local path = require("luarocks.path")
+local rep = require("luarocks.rep")
+local deps = require("luarocks.deps")
 
-manifest_cache = {}
+rock_manifest_cache = {}
 
---- Get all versions of a package listed in a manifest file.
--- @param name string: a package name.
--- @param manifest table or nil: a manifest table; if not given, the
--- default local manifest table is used.
--- @return table: An array of strings listing installed
--- versions of a package.
-function get_versions(name, manifest)
+--- Commit a table to disk in given local path.
+-- @param where string: The directory where the table should be saved.
+-- @param name string: The filename.
+-- @param tbl table: The table to be saved.
+-- @return boolean or (nil, string): true if successful, or nil and a
+-- message in case of errors.
+local function save_table(where, name, tbl)
+   assert(type(where) == "string")
    assert(type(name) == "string")
-   assert(type(manifest) == "table" or not manifest)
-   
-   if not manifest then
-      manifest = load_local_manifest(cfg.rocks_dir)
-      if not manifest then
-         return {}
-      end
-   end
-   
-   local item = manifest.repository[name]
-   if item then
-      return util.keys(item)
-   end
-   return {}
+   assert(type(tbl) == "table")
+
+   local filename = dir.path(where, name)
+   return persist.save_from_table(filename, tbl)
 end
 
---- Back-end function that actually loads the manifest
--- and stores it in the manifest cache.
--- @param file string: The local filename of the manifest file.
--- @param repo_url string: The repository identifier.
-local function manifest_loader(file, repo_url, quick)
-   local manifest = persist.load_into_table(file)
-   if not manifest then
-      return nil, "Failed loading manifest for "..repo_url
+function load_rock_manifest(name, version)
+   assert(type(name) == "string")
+   assert(type(version) == "string")
+
+   local name_version = name.."/"..version
+   if rock_manifest_cache[name_version] then
+      return rock_manifest_cache[name_version].rock_manifest
    end
-   if not quick then
-      local ok, err = type_check.type_check_manifest(manifest)
-      if not ok then
-         return nil, "Error checking manifest: "..err
+   local pathname = path.rock_manifest_file(name, version)
+   local rock_manifest = persist.load_into_table(pathname)
+   if not rock_manifest then return nil end
+   rock_manifest_cache[name_version] = rock_manifest
+   return rock_manifest.rock_manifest
+end
+
+function make_rock_manifest(name, version)
+   local install_dir = path.install_dir(name, version)
+   local rock_manifest = path.rock_manifest_file(name, version)
+   local tree = {}
+   for _, file in ipairs(fs.find(install_dir)) do
+      local full_path = dir.path(install_dir, file)
+      local walk = tree
+      local last
+      local last_name
+      for name in file:gmatch("[^/]+") do
+         local next = walk[name]
+         if not next then 
+            next = {}
+            walk[name] = next
+         end
+         last = walk
+         last_name = name
+         walk = next
+      end
+      if fs.is_file(full_path) then
+         last[last_name] = fs.get_md5(full_path)
       end
    end
-
-   manifest_cache[repo_url] = manifest
-   return manifest
+   local rock_manifest = { rock_manifest=tree }
+   rock_manifest_cache[name.."/"..version] = rock_manifest
+   save_table(install_dir, "rock_manifest", rock_manifest )
 end
 
 --- Load a local or remote manifest describing a repository.
 -- All functions that use manifest tables assume they were obtained
 -- through either this function or load_local_manifest.
 -- @param repo_url string: URL or pathname for the repository.
--- @return table or (nil, string): A table representing the manifest,
--- or nil followed by an error message.
+-- @return table or (nil, string, [string]): A table representing the manifest,
+-- or nil followed by an error message and an optional error code.
 function load_manifest(repo_url)
    assert(type(repo_url) == "string")
    
-   if manifest_cache[repo_url] then
-      return manifest_cache[repo_url]
+   if manif_core.manifest_cache[repo_url] then
+      return manif_core.manifest_cache[repo_url]
    end
 
-   local protocol, pathname = fs.split_url(repo_url)
+   local protocol, pathname = dir.split_url(repo_url)
    if protocol == "file" then
-      pathname = fs.make_path(pathname, "manifest")
+      pathname = dir.path(pathname, "manifest")
    else
-      local url = fs.make_path(repo_url, "manifest")
+      local url = dir.path(repo_url, "manifest")
       local name = repo_url:gsub("[/:]","_")
-      local file, dir = fetch.fetch_url_at_temp_dir(url, "luarocks-manifest-"..name)
+      local file, err, errcode = fetch.fetch_url_at_temp_dir(url, "luarocks-manifest-"..name)
       if not file then
-         return nil, "Failed fetching manifest for "..repo_url
+         return nil, "Failed fetching manifest for "..repo_url, errcode
       end
       pathname = file
    end
-   return manifest_loader(pathname, repo_url)
+   return manif_core.manifest_loader(pathname, repo_url)
 end
 
---- Load a local manifest describing a repository.
--- All functions that use manifest tables assume they were obtained
--- through either this function or load_manifest.
--- @param repo_url string: URL or pathname for the repository.
--- @return table or (nil, string): A table representing the manifest,
--- or nil followed by an error message.
-function load_local_manifest(repo_url)
-   assert(type(repo_url) == "string")
+--- Output a table listing items of a package.
+-- @param itemsfn function: a function for obtaining items of a package.
+-- pkg and version will be passed to it; it should return a table with
+-- items as keys.
+-- @param pkg string: package name
+-- @param version string: package version
+-- @param tbl table: the package matching table: keys should be item names
+-- and values arrays of strings with packages names in "name/version" format.
+local function store_package_items(itemsfn, pkg, version, tbl)
+   assert(type(itemsfn) == "function")
+   assert(type(pkg) == "string")
+   assert(type(version) == "string")
+   assert(type(tbl) == "table")
 
-   if manifest_cache[repo_url] then
-      return manifest_cache[repo_url]
+   local pkg_version = pkg.."/"..version
+   local result = {}
+
+   for item, path in pairs(itemsfn(pkg, version)) do
+      result[item] = path
+      if not tbl[item] then
+         tbl[item] = {}
+      end
+      table.insert(tbl[item], pkg_version)
    end
-
-   local pathname = fs.make_path(repo_url, "manifest")
-
-   return manifest_loader(pathname, repo_url, true)
+   return result
 end
 
 --- Sort function for ordering rock identifiers in a manifest's
@@ -121,32 +145,6 @@ local function sort_pkgs(a, b)
    return (na == nb) and deps.compare_versions(va, vb) or na < nb
 end
 
---- Output a table listing items of a package.
--- @param itemsfn function: a function for obtaining items of a package.
--- pkg and version will be passed to it; it should return a table with
--- items as keys.
--- @param pkg string: package name
--- @param version string: package version
--- @param tbl table: the package matching table: keys should be item names
--- and values arrays of strings with packages names in "name/version" format.
-local function store_package_items(itemsfn, pkg, version, tbl)
-   assert(type(itemsfn) == "function")
-   assert(type(pkg) == "string")
-   assert(type(version) == "string")
-   assert(type(tbl) == "table")
-
-   local path = pkg.."/"..version
-   local result = {}
-   for item, _ in pairs(itemsfn(pkg, version)) do
-      table.insert(result, item)
-      if not tbl[item] then
-         tbl[item] = {}
-      end
-      table.insert(tbl[item], path)
-   end
-   return result
-end
-
 --- Sort items of a package matching table by version number (higher versions first).
 -- @param tbl table: the package matching table: keys should be strings
 -- and values arrays of strings with packages names in "name/version" format.
@@ -174,19 +172,6 @@ local function sort_package_matching_table(tbl)
    end
 end
 
---- Commit manifest to disk in given local repository.
--- @param repo string: The directory of the local repository.
--- @param manifest table: The manifest table
--- @return boolean or (nil, string): true if successful, or nil and a
--- message in case of errors.
-local function save_manifest(repo, manifest)
-   assert(type(repo) == "string")
-   assert(type(manifest) == "table")
-
-   local filename = fs.make_path(repo, "manifest")
-   return persist.save_from_table(filename, manifest)
-end
-
 --- Process the dependencies of a package to determine its dependency
 -- chain for loading modules.
 -- @param name string: Package name.
@@ -224,26 +209,57 @@ local function store_results(results, manifest)
    assert(type(results) == "table")
    assert(type(manifest) == "table")
 
-   for pkg, versions in pairs(results) do
-      local pkgtable = manifest.repository[pkg] or {}
-      for version, repos in pairs(versions) do
+   for name, versions in pairs(results) do
+      local pkgtable = manifest.repository[name] or {}
+      for version, entries in pairs(versions) do
          local versiontable = {}
-         for _, repo in ipairs(repos) do
-            local repotable = {}
-            repotable.arch = repo.arch
-            if repo.arch == "installed" then
-               repotable.modules = store_package_items(rep.package_modules, pkg, version, manifest.modules)
-               repotable.commands = store_package_items(rep.package_commands, pkg, version, manifest.commands)
+         for _, entry in ipairs(entries) do
+            local entrytable = {}
+            entrytable.arch = entry.arch
+            if entry.arch == "installed" then
+               local rock_manifest = load_rock_manifest(name, version)
+               if not rock_manifest then
+                  return nil, "rock_manifest file not found for "..name.." "..version.." - not a LuaRocks 2 tree?"
+               end
+               entrytable.modules = store_package_items(rep.package_modules, name, version, manifest.modules)
+               entrytable.commands = store_package_items(rep.package_commands, name, version, manifest.commands)
             end
-            table.insert(versiontable, repotable)
+            table.insert(versiontable, entrytable)
          end
          pkgtable[version] = versiontable
       end
-      manifest.repository[pkg] = pkgtable
+      manifest.repository[name] = pkgtable
    end
    update_dependencies(manifest)
    sort_package_matching_table(manifest.modules)
    sort_package_matching_table(manifest.commands)
+   return true
+end
+
+--- Scan a LuaRocks repository and output a manifest file.
+-- A file called 'manifest' will be written in the root of the given
+-- repository directory.
+-- @param repo A local repository directory.
+-- @return boolean or (nil, string): True if manifest was generated,
+-- or nil and an error message.
+function make_manifest(repo)
+   assert(type(repo) == "string")
+
+   if not fs.is_dir(repo) then
+      return nil, "Cannot access repository at "..repo
+   end
+
+   local query = search.make_query("")
+   query.exact_name = false
+   query.arch = "any"
+   local results = search.disk_search(repo, query)
+   local manifest = { repository = {}, modules = {}, commands = {} }
+   manif_core.manifest_cache[repo] = manifest
+
+   local ok, err = store_results(results, manifest)
+   if not ok then return nil, err end
+
+   return save_table(repo, "manifest", manifest)
 end
 
 --- Load a manifest file from a local repository and add to the repository
@@ -278,162 +294,68 @@ function update_manifest(name, version, repo)
    end
 
    local results = {[name] = {[version] = {{arch = "installed", repo = repo}}}}
+
+   local ok, err = store_results(results, manifest)
+   if not ok then return nil, err end
    
-   store_results(results, manifest)
-   return save_manifest(repo, manifest)
+   return save_table(repo, "manifest", manifest)
 end
 
---- Scan a LuaRocks repository and output a manifest file.
--- A file called 'manifest' will be written in the root of the given
--- repository directory.
--- @param repo A local repository directory.
--- @return boolean or (nil, string): True if manifest was generated,
--- or nil and an error message.
-function make_manifest(repo)
-   assert(type(repo) == "string")
-
-   if not fs.is_dir(repo) then
-      return nil, "Cannot access repository at "..repo
-   end
-
-   local query = search.make_query("")
-   query.exact_name = false
-   query.arch = "any"
-   local results = search.disk_search(repo, query)
-
-   local manifest = { repository = {}, modules = {}, commands = {} }
-   manifest_cache[repo] = manifest
-   store_results(results, manifest)
-   return save_manifest(repo, manifest)
+local function starts_with(s, prefix)
+   return s:sub(1,#prefix) == prefix
 end
 
-local index_header = [[
-<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
-<html>
-<head>
-<title>Available rocks</title>
-<meta http-equiv="content-type" content="text/html; charset=iso-8859-1">
-<style>
-body {
-   background-color: white;
-   font-family: "bitstream vera sans", "verdana", "sans";
-   font-size: 14px;
-}
-a {
-   color: #0000c0;
-   text-decoration: none;
-}
-a:hover {
-   text-decoration: underline;
-}
-td.main {
-   border-style: none;
-}
-blockquote {
-   font-size: 12px;
-}
-td.package {
-   background-color: #f0f0f0;
-   vertical-align: top;
-}
-td.spacer {
-   height: 5px;
-}
-td.version {
-   background-color: #d0d0d0;
-   vertical-align: top;
-   text-align: left;
-   padding: 5px;
-   width: 100px;
-}
-p.manifest {
-   font-size: 8px;
-}
-</style>
-</head>
-<body>
-<h1>Available rocks</h1>
-<p>
-Lua modules avaliable from this location for use with <a href="http://www.luarocks.org">LuaRocks</a>:
-</p>
-<table class="main">
-]]
+local function find_providers(file, root)
+   assert(type(file) == "string")
+   assert(type(root) == "string" or not root)
+   root = root or cfg.root_dir
 
-local index_package_start = [[
-<td class="package">
-<p><a name="$anchor"></a><b>$package</b> - $summary<br/>
-</p><blockquote><p>$detailed<br/>
-<font size="-1"><a href="$original">latest sources</a> | <a href="$homepage">project homepage</a> | License: $license</font></p>
-</blockquote></a></td>
-<td class="version">
-]]
-
-local index_package_end = [[
-</td></tr>
-<tr><td colspan="2" class="spacer"></td></tr>
-]]
-
-local index_footer = [[
-</table>
-<p class="manifest">
-<a href="manifest">manifest file</a>
-</p>
-</body>
-</html>
-]]
-
-function make_index(repo)
-   if not fs.is_dir(repo) then
-      return nil, "Cannot access repository at "..repo
+   local manifest = manif_core.load_local_manifest(path.rocks_dir(root))
+   if not manifest then
+      return nil, "manifest file is missing. Corrupted local rocks tree?"
    end
-   local manifest = load_manifest(repo)
-   files = fs.find(repo)
-   local out = io.open(fs.make_path(repo, "index.html"), "w")
-   out:write(index_header)
-   for package, version_list in util.sortedpairs(manifest.repository) do
-      local latest_rockspec = nil
-      local output = index_package_start
-      for version, data in util.sortedpairs(version_list, deps.compare_versions) do
-         local out_versions = {}
-         local arches = 0
-         output = output..version
-         local sep = ':&nbsp;'
-         for _, item in ipairs(data) do
-            output = output .. sep .. '<a href="$url">'..item.arch..'</a>'
-            sep = ',&nbsp;'
-            if item.arch == 'rockspec' then
-               local rs = ("%s-%s.rockspec"):format(package, version)
-               if not latest_rockspec then latest_rockspec = rs end
-               output = output:gsub("$url", rs)
-            else
-               output = output:gsub("$url", ("%s-%s.%s.rock"):format(package, version, item.arch))
-            end
-         end
-         output = output .. '<br/>'
-         output = output:gsub("$na", arches)
-      end
-      output = output .. index_package_end
-      if latest_rockspec then
-         local rockspec = persist.load_into_table(fs.make_path(repo, latest_rockspec))
-         local vars = {
-            anchor = package,
-            package = rockspec.package,
-            original = rockspec.source.url,
-            summary = rockspec.description.summary or "",
-            detailed = rockspec.description.detailed or "",
-            license = rockspec.description.license or "N/A",
-            homepage = rockspec.description.homepage or ""
-         }
-         vars.detailed = vars.detailed:gsub("\n\n", "</p><p>"):gsub("%s+", " ")
-         output = output:gsub("$(%w+)", vars)
-      else
-         output = output:gsub("$anchor", package)
-         output = output:gsub("$package", package)
-         output = output:gsub("$(%w+)", "")
-      end
-      out:write(output)
+   local deploy_bin = path.deploy_bin_dir(root)
+   local deploy_lua = path.deploy_lua_dir(root)
+   local deploy_lib = path.deploy_lib_dir(root)
+   local key, manifest_tbl
+
+   if starts_with(file, deploy_bin) then
+      manifest_tbl = manifest.commands
+      key = file:sub(#deploy_bin+1)
+   elseif starts_with(file, deploy_lua) then
+      manifest_tbl = manifest.modules
+      key = path.path_to_module(file:sub(#deploy_lua+1))
+   elseif starts_with(file, deploy_lib) then
+      manifest_tbl = manifest.modules
+      key = path.path_to_module(file:sub(#deploy_lib+1))
+   else
+      assert(false, "Assertion failed: '"..file.."' is not a deployed file.")
    end
-   out:write(index_footer)
-   out:close()
+
+   local providers = manifest_tbl[key]
+   if not providers then
+      return nil, "File "..file.." is not tracked by LuaRocks."
+   end
+   return providers
 end
 
+--- Given a path of a deployed file, figure out which rock name and version
+-- correspond to it in the tree manifest.
+-- @param file string: The full path of a deployed file.
+-- @param root string or nil: A local root dir for a rocks tree. If not given, the default is used.
+-- @return string, string: name and version of the provider rock.
+function find_current_provider(file, root)
+   local providers, err = find_providers(file, root)
+   if not providers then return nil, err end
+   return providers[1]:match("([^/]*)/([^/]*)")
+end
+
+function find_next_provider(file, root)
+   local providers, err = find_providers(file, root)
+   if not providers then return nil, err end
+   if providers[2] then
+      return providers[2]:match("([^/]*)/([^/]*)")
+   else
+      return nil
+   end
+end
